# lingo - payment and billing system
# Copyright (C) 2022  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.forms import ValidationError
from django.template import Template, TemplateSyntaxError
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from lingo.agendas.models import CheckType
from lingo.pricing.models import AgendaPricing, Criteria, CriteriaCategory


class ExportForm(forms.Form):
    agendas = forms.BooleanField(label=_('Agendas'), required=False, initial=True)
    check_type_groups = forms.BooleanField(label=_('Check type groups'), required=False, initial=True)
    pricing_categories = forms.BooleanField(
        label=_('Pricing criteria categories'), required=False, initial=True
    )
    pricing_models = forms.BooleanField(label=_('Pricing models'), required=False, initial=True)


class ImportForm(forms.Form):
    config_json = forms.FileField(label=_('Export File'))


class NewCriteriaForm(forms.ModelForm):
    class Meta:
        model = Criteria
        fields = ['label', 'default', 'condition']

    def clean(self):
        cleaned_data = super().clean()

        if cleaned_data.get('default') is True:
            cleaned_data['condition'] = ''
        else:
            condition = cleaned_data['condition']
            if not condition:
                self.add_error('condition', self.fields['condition'].default_error_messages['required'])
            else:
                try:
                    Template('{%% if %s %%}OK{%% endif %%}' % condition)
                except TemplateSyntaxError:
                    self.add_error('condition', _('Invalid syntax.'))

        return cleaned_data


class CriteriaForm(NewCriteriaForm):
    class Meta:
        model = Criteria
        fields = ['label', 'slug', 'default', 'condition']

    def clean_slug(self):
        slug = self.cleaned_data['slug']

        if self.instance.category.criterias.filter(slug=slug).exclude(pk=self.instance.pk).exists():
            raise ValidationError(_('Another criteria exists with the same identifier.'))

        return slug


class PricingDuplicateForm(forms.Form):
    label = forms.CharField(label=_('New label'), max_length=150, required=False)


class PricingVariableForm(forms.Form):
    key = forms.CharField(label=_('Variable name'), required=False)
    value = forms.CharField(
        label=_('Value template'), widget=forms.TextInput(attrs={'size': 60}), required=False
    )


PricingVariableFormSet = forms.formset_factory(PricingVariableForm)


class PricingCriteriaCategoryAddForm(forms.Form):
    category = forms.ModelChoiceField(
        label=_('Criteria category to add'), queryset=CriteriaCategory.objects.none(), required=True
    )

    def __init__(self, *args, **kwargs):
        self.pricing = kwargs.pop('pricing')
        super().__init__(*args, **kwargs)
        self.fields['category'].queryset = CriteriaCategory.objects.exclude(pricings=self.pricing)


class PricingCriteriaCategoryEditForm(forms.Form):
    criterias = forms.ModelMultipleChoiceField(
        label=_('Criterias'),
        queryset=Criteria.objects.none(),
        required=True,
        widget=forms.CheckboxSelectMultiple(),
    )

    def __init__(self, *args, **kwargs):
        self.pricing = kwargs.pop('pricing')
        self.category = kwargs.pop('category')
        super().__init__(*args, **kwargs)
        self.fields['criterias'].queryset = self.category.criterias.all()
        self.initial['criterias'] = self.pricing.criterias.filter(category=self.category)


class AgendaPricingForm(forms.ModelForm):
    class Meta:
        model = AgendaPricing
        fields = ['pricing', 'date_start', 'date_end']
        widgets = {
            'date_start': forms.DateInput(attrs={'type': 'date'}, format='%Y-%m-%d'),
            'date_end': forms.DateInput(attrs={'type': 'date'}, format='%Y-%m-%d'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self.instance.pk:
            last_date_end = (
                AgendaPricing.objects.filter(agenda=self.instance.agenda)
                .order_by('date_end')
                .values_list('date_end', flat=True)
                .last()
            )
            self.initial['date_start'] = last_date_end or now().date()

    def clean(self):
        cleaned_data = super().clean()

        if 'date_start' in cleaned_data and 'date_end' in cleaned_data:
            if cleaned_data['date_end'] <= cleaned_data['date_start']:
                self.add_error('date_end', _('End date must be greater than start date.'))
            else:
                overlapping_qs = AgendaPricing.objects.filter(agenda=self.instance.agenda,).extra(
                    where=["(date_start, date_end) OVERLAPS (%s, %s)"],
                    params=[cleaned_data['date_start'], cleaned_data['date_end']],
                )
                if self.instance.pk:
                    overlapping_qs = overlapping_qs.exclude(pk=self.instance.pk)
                if overlapping_qs.exists():
                    raise forms.ValidationError(_('Pricing overlaps existing pricings.'))

        return cleaned_data


class PricingMatrixForm(forms.Form):
    def __init__(self, *args, **kwargs):
        matrix = kwargs.pop('matrix')
        super().__init__(*args, **kwargs)
        for i in range(len(matrix.rows[0].cells)):
            self.fields['crit_%i' % i] = forms.DecimalField(required=True, max_digits=5, decimal_places=2)


class NewCheckTypeForm(forms.ModelForm):
    class Meta:
        model = CheckType
        fields = ['label', 'kind', 'pricing', 'pricing_rate']

    def clean(self):
        super().clean()
        if self.cleaned_data.get('pricing') is not None and self.cleaned_data.get('pricing_rate') is not None:
            raise ValidationError(_('Please choose between pricing and pricing rate.'))


class CheckTypeForm(NewCheckTypeForm):
    class Meta:
        model = CheckType
        fields = ['label', 'slug', 'pricing', 'pricing_rate', 'disabled']

    def clean_slug(self):
        slug = self.cleaned_data['slug']

        if self.instance.group.check_types.filter(slug=slug).exclude(pk=self.instance.pk).exists():
            raise ValidationError(_('Another check type exists with the same identifier.'))

        return slug
